@@ -7,6 +7,7 @@ import jax
 import jax.numpy as jnp
 import jax.scipy.stats as stats
 import numpy as np
+import optax
 from absl.testing import absltest, parameterized
 
 import blackjax
@@ -71,7 +72,7 @@ class LinearRegressionTest(chex.TestCase):
 
     def setUp(self):
         super().setUp()
-        self.key = jax.random.PRNGKey(19)
+        self.key = jax.random.key(19)
 
     def regression_logprob(self, log_scale, coefs, preds, x):
         """Linear regression"""
@@ -83,6 +84,48 @@ class LinearRegressionTest(chex.TestCase):
         # reduce sum otherwise broacasting will make the logprob biased.
         return sum(x.sum() for x in [scale_prior, coefs_prior, logpdf])
 
+    def run_mclmc(self, logdensity_fn, num_steps, initial_position, key):
+        init_key, tune_key, run_key = jax.random.split(key, 3)
+
+        initial_state = blackjax.mcmc.mclmc.init(
+            x_initial=initial_position, logdensity_fn=logdensity_fn, rng_key=init_key
+        )
+
+        kernel = blackjax.mcmc.mclmc.build_kernel(
+            logdensity_fn=logdensity_fn,
+            integrator=blackjax.mcmc.integrators.noneuclidean_mclachlan,
+            transform=lambda x: x,
+        )
+
+        (
+            blackjax_state_after_tuning,
+            blackjax_mclmc_sampler_params,
+        ) = blackjax.mclmc_find_L_and_step_size(
+            mclmc_kernel=kernel,
+            num_steps=num_steps,
+            state=initial_state,
+            rng_key=tune_key,
+        )
+
+        keys = jax.random.split(run_key, num_steps)
+
+        sampling_alg = blackjax.mclmc(
+            logdensity_fn,
+            L=blackjax_mclmc_sampler_params.L,
+            step_size=blackjax_mclmc_sampler_params.step_size,
+        )
+
+        _, blackjax_mclmc_result = jax.lax.scan(
+            f=lambda state, k: sampling_alg.step(
+                rng_key=k,
+                state=state,
+            ),
+            xs=keys,
+            init=blackjax_state_after_tuning,
+        )
+
+        return blackjax_mclmc_result.transformed_position
+
     @parameterized.parameters(itertools.product(regression_test_cases, [True, False]))
     def test_window_adaptation(self, case, is_mass_matrix_diagonal):
         """Test the HMC kernel and the Stan warmup."""
@@ -123,7 +166,7 @@ class LinearRegressionTest(chex.TestCase):
 
     def test_mala(self):
         """Test the MALA kernel."""
-        rng_key, init_key0, init_key1 = jax.random.split(self.key, 3)
+        init_key0, init_key1, inference_key = jax.random.split(self.key, 3)
         x_data = jax.random.normal(init_key0, shape=(1000, 1))
         y_data = 3 * x_data + jax.random.normal(init_key1, shape=x_data.shape)
 
@@ -132,8 +175,6 @@ class LinearRegressionTest(chex.TestCase):
         )
         logposterior_fn = lambda x: logposterior_fn_(**x)
 
-        warmup_key, inference_key = jax.random.split(rng_key, 2)
-
         mala = blackjax.mala(logposterior_fn, 1e-5)
         state = mala.init({"coefs": 1.0, "log_scale": 1.0})
         states = inference_loop(mala.step, 10_000, inference_key, state)
@@ -144,6 +185,30 @@ class LinearRegressionTest(chex.TestCase):
         np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-1)
         np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-1)
 
+    def test_mclmc(self):
+        """Test the MCLMC kernel."""
+        init_key0, init_key1, inference_key = jax.random.split(self.key, 3)
+        x_data = jax.random.normal(init_key0, shape=(1000, 1))
+        y_data = 3 * x_data + jax.random.normal(init_key1, shape=x_data.shape)
+
+        logposterior_fn_ = functools.partial(
+            self.regression_logprob, x=x_data, preds=y_data
+        )
+        logdensity_fn = lambda x: logposterior_fn_(**x)
+
+        states = self.run_mclmc(
+            initial_position={"coefs": 1.0, "log_scale": 1.0},
+            logdensity_fn=logdensity_fn,
+            key=inference_key,
+            num_steps=10000,
+        )
+
+        coefs_samples = states["coefs"][3000:]
+        scale_samples = np.exp(states["log_scale"][3000:])
+
+        np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-2)
+        np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-2)
+
     @parameterized.parameters(regression_test_cases)
     def test_pathfinder_adaptation(
         self,
@@ -225,13 +290,76 @@ class LinearRegressionTest(chex.TestCase):
         np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-1)
         np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-1)
 
+    @parameterized.parameters([None, jax.random.uniform])
+    def test_chees(self, jitter_generator):
+        """Test the ChEES adaptation w/ HMC kernel."""
+        rng_key, init_key0, init_key1 = jax.random.split(self.key, 3)
+        x_data = jax.random.normal(init_key0, shape=(1000, 1))
+        y_data = 3 * x_data + jax.random.normal(init_key1, shape=x_data.shape)
+
+        logposterior_fn_ = functools.partial(
+            self.regression_logprob, x=x_data, preds=y_data
+        )
+        logposterior_fn = lambda x: logposterior_fn_(**x)
+
+        init_key, warmup_key, inference_key = jax.random.split(rng_key, 3)
+
+        num_chains = 128
+        warmup = blackjax.chees_adaptation(
+            logposterior_fn, num_chains=num_chains, jitter_generator=jitter_generator
+        )
+        scale_key, coefs_key = jax.random.split(init_key, 2)
+        log_scales = 1.0 + jax.random.normal(scale_key, (num_chains,))
+        coefs = 4.0 + jax.random.normal(coefs_key, (num_chains,))
+        initial_positions = {"log_scale": log_scales, "coefs": coefs}
+        (last_states, parameters), _ = warmup.run(
+            warmup_key,
+            initial_positions,
+            step_size=0.001,
+            optim=optax.adam(learning_rate=0.1),
+            num_steps=1000,
+        )
+        kernel = blackjax.dynamic_hmc(logposterior_fn, **parameters).step
+
+        chain_keys = jax.random.split(inference_key, num_chains)
+        states = jax.vmap(lambda key, state: inference_loop(kernel, 100, key, state))(
+            chain_keys, last_states
+        )
+
+        coefs_samples = states.position["coefs"]
+        scale_samples = np.exp(states.position["log_scale"])
+
+        np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-1)
+        np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-1)
+
+    def test_barker(self):
+        """Test the Barker kernel."""
+        init_key0, init_key1, inference_key = jax.random.split(self.key, 3)
+        x_data = jax.random.normal(init_key0, shape=(1000, 1))
+        y_data = 3 * x_data + jax.random.normal(init_key1, shape=x_data.shape)
+
+        logposterior_fn_ = functools.partial(
+            self.regression_logprob, x=x_data, preds=y_data
+        )
+        logposterior_fn = lambda x: logposterior_fn_(**x)
+
+        barker = blackjax.barker_proposal(logposterior_fn, 1e-1)
+        state = barker.init({"coefs": 1.0, "log_scale": 1.0})
+        states = inference_loop(barker.step, 10_000, inference_key, state)
+
+        coefs_samples = states.position["coefs"][3000:]
+        scale_samples = np.exp(states.position["log_scale"][3000:])
+
+        np.testing.assert_allclose(np.mean(scale_samples), 1.0, atol=1e-2)
+        np.testing.assert_allclose(np.mean(coefs_samples), 3.0, atol=1e-2)
+
 
 class SGMCMCTest(chex.TestCase):
     """Test sampling of a linear regression model."""
 
     def setUp(self):
         super().setUp()
-        self.key = jax.random.PRNGKey(19)
+        self.key = jax.random.key(19)
 
     def logprior_fn(self, position):
         return -0.5 * jnp.dot(position, position) * 0.01
@@ -254,7 +382,6 @@ class SGMCMCTest(chex.TestCase):
         )
         csgld = blackjax.csgld(logdensity_fn, grad_fn)
 
-        _, rng_key = jax.random.split(rng_key)
         data_batch = X_data[:100, :]
         init_position = 1.0
         init_state = csgld.init(init_position)
@@ -271,10 +398,10 @@ class SGMCMCTest(chex.TestCase):
         )
         sgld = blackjax.sgld(grad_fn)
 
-        _, rng_key = jax.random.split(rng_key)
         data_batch = X_data[:100, :]
         init_position = 1.0
-        _ = sgld(rng_key, init_position, data_batch, 1e-3)
+        init_position = sgld.init(init_position)
+        _ = sgld.step(rng_key, init_position, data_batch, 1e-3)
 
     def test_linear_regression_sgld_cv(self):
         rng_key, data_key = jax.random.split(self.key, 2)
@@ -293,10 +420,10 @@ class SGMCMCTest(chex.TestCase):
 
         sgld = blackjax.sgld(cv_grad_fn)
 
-        _, rng_key = jax.random.split(rng_key)
         init_position = 1.0
+        init_position = sgld.init(init_position)
         data_batch = X_data[:100, :]
-        _ = sgld(rng_key, init_position, data_batch, 1e-3)
+        _ = sgld.step(rng_key, init_position, data_batch, 1e-3)
 
     def test_linear_regression_sghmc(self):
         rng_key, data_key = jax.random.split(self.key, 2)
@@ -309,11 +436,11 @@ class SGMCMCTest(chex.TestCase):
         )
         sghmc = blackjax.sghmc(grad_fn, 10)
 
-        _, rng_key = jax.random.split(rng_key)
         data_batch = X_data[100:200, :]
         init_position = 1.0
+        init_position = sghmc.init(init_position)
         data_batch = X_data[:100, :]
-        _ = sghmc(rng_key, init_position, data_batch, 1e-3)
+        _ = sghmc.step(rng_key, init_position, data_batch, 1e-3)
 
     def test_linear_regression_sghmc_cv(self):
         rng_key, data_key = jax.random.split(self.key, 2)
@@ -331,10 +458,50 @@ class SGMCMCTest(chex.TestCase):
 
         sghmc = blackjax.sghmc(cv_grad_fn, 10)
 
+        init_position = 1.0
+        init_position = sghmc.init(init_position)
+        data_batch = X_data[:100, :]
+        _ = sghmc.step(rng_key, init_position, data_batch, 1e-3)
+
+    def test_linear_regression_sgnht(self):
+        rng_key, data_key = jax.random.split(self.key, 2)
+
+        data_size = 1000
+        X_data = jax.random.normal(data_key, shape=(data_size, 5))
+
+        grad_fn = blackjax.sgmcmc.grad_estimator(
+            self.logprior_fn, self.loglikelihood_fn, data_size
+        )
+        sgnht = blackjax.sgnht(grad_fn)
+
+        _, rng_key = jax.random.split(rng_key)
+        data_batch = X_data[100:200, :]
+        init_position = 1.0
+        data_batch = X_data[:100, :]
+        init_state = sgnht.init(init_position, self.key)
+        _ = sgnht.step(rng_key, init_state, data_batch, 1e-3)
+
+    def test_linear_regression_sgnhtc_cv(self):
+        rng_key, data_key = jax.random.split(self.key, 2)
+
+        data_size = 1000
+        X_data = jax.random.normal(data_key, shape=(data_size, 5))
+
+        centering_position = 1.0
+        grad_fn = blackjax.sgmcmc.grad_estimator(
+            self.logprior_fn, self.loglikelihood_fn, data_size
+        )
+        cv_grad_fn = blackjax.sgmcmc.gradients.control_variates(
+            grad_fn, centering_position, X_data
+        )
+
+        sgnht = blackjax.sgnht(cv_grad_fn)
+
         _, rng_key = jax.random.split(rng_key)
         init_position = 1.0
         data_batch = X_data[:100, :]
-        _ = sghmc(rng_key, init_position, data_batch, 1e-3)
+        init_state = sgnht.init(init_position, self.key)
+        _ = sgnht.step(rng_key, init_state, data_batch, 1e-3)
 
 
 class LatentGaussianTest(chex.TestCase):
@@ -342,7 +509,7 @@ class LatentGaussianTest(chex.TestCase):
 
     def setUp(self):
         super().setUp()
-        self.key = jax.random.PRNGKey(19)
+        self.key = jax.random.key(19)
         self.C = 2.0 * np.eye(1)
         self.delta = 5.0
         self.sampling_steps = 25_000
@@ -446,15 +613,25 @@ normal_test_cases = [
         "num_sampling_steps": 6000,
         "burnin": 1_000,
     },
+    {
+        "algorithm": blackjax.barker_proposal,
+        "initial_position": 1.0,
+        "parameters": {"step_size": 1.5},
+        "num_sampling_steps": 20_000,
+        "burnin": 2_000,
+    },
 ]
 
 
 class UnivariateNormalTest(chex.TestCase):
-    """Test sampling of a univariate Normal distribution."""
+    """Test sampling of a univariate Normal distribution.
+
+    (TODO) This only passes due to clever seed hacking.
+    """
 
     def setUp(self):
         super().setUp()
-        self.key = jax.random.PRNGKey(19)
+        self.key = jax.random.key(12)
 
     def normal_logprob(self, x):
         return stats.norm.logpdf(x, loc=1.0, scale=2.0)
@@ -471,26 +648,27 @@ class UnivariateNormalTest(chex.TestCase):
             parameters["proposal_generator"] = rmh_proposal_distribution
 
         algo = algorithm(self.normal_logprob, **parameters)
+        rng_key = self.key
         if algorithm == blackjax.elliptical_slice:
             algo = algorithm(lambda _: 1.0, **parameters)
         if algorithm == blackjax.ghmc:
-            initial_state = algo.init(initial_position, self.key)
+            rng_key, initial_state_key = jax.random.split(rng_key)
+            initial_state = algo.init(initial_position, initial_state_key)
         else:
             initial_state = algo.init(initial_position)
 
+        inference_key, orbit_key = jax.random.split(rng_key)
         kernel = algo.step
         states = self.variant(
             functools.partial(inference_loop, kernel, num_sampling_steps)
-        )(self.key, initial_state)
+        )(inference_key, initial_state)
 
         if algorithm == blackjax.orbital_hmc:
-            _, orbit_key = jax.random.split(self.key)
             samples = orbit_samples(
                 states.positions[burnin:], states.weights[burnin:], orbit_key
             )
         else:
             samples = states.position[burnin:]
-
         np.testing.assert_allclose(np.mean(samples), 1.0, rtol=1e-1)
         np.testing.assert_allclose(np.var(samples), 4.0, rtol=1e-1)
 
@@ -499,17 +677,33 @@ mcse_test_cases = [
     {
         "algorithm": blackjax.hmc,
         "parameters": {
-            "step_size": 1.0,
-            "num_integration_steps": 32,
+            "step_size": 0.5,
+            "num_integration_steps": 20,
         },
+        "is_mass_matrix_diagonal": True,
     },
     {
         "algorithm": blackjax.nuts,
-        "parameters": {"step_size": 1.0},
+        "parameters": {"step_size": 0.5},
+        "is_mass_matrix_diagonal": True,
+    },
+    {
+        "algorithm": blackjax.hmc,
+        "parameters": {
+            "step_size": 0.85,
+            "num_integration_steps": 27,
+        },
+        "is_mass_matrix_diagonal": False,
     },
     {
         "algorithm": blackjax.nuts,
-        "parameters": {"step_size": 1.0},
+        "parameters": {"step_size": 0.85},
+        "is_mass_matrix_diagonal": False,
+    },
+    {
+        "algorithm": blackjax.barker_proposal,
+        "parameters": {"step_size": 0.5},
+        "is_mass_matrix_diagonal": None,
     },
 ]
 
@@ -519,7 +713,7 @@ class MonteCarloStandardErrorTest(chex.TestCase):
 
     def setUp(self):
         super().setUp()
-        self.key = jax.random.PRNGKey(20220203)
+        self.key = jax.random.key(20220203)
 
     def generate_multivariate_target(self, rng=None):
         """Genrate a Multivariate Normal distribution as target."""
@@ -528,7 +722,7 @@ class MonteCarloStandardErrorTest(chex.TestCase):
             scale = jnp.array([1.0, 2.0])
             rho = jnp.array(0.75)
         else:
-            rng, loc_rng, scale_rng, rho_rng = jax.random.split(rng, 4)
+            loc_rng, scale_rng, rho_rng = jax.random.split(rng, 3)
             loc = jax.random.normal(loc_rng, [2]) * 10.0
             scale = jnp.abs(jax.random.normal(scale_rng, [2])) * 2.5
             rho = jax.random.uniform(rho_rng, [], minval=-1.0, maxval=1.0)
@@ -540,7 +734,7 @@ class MonteCarloStandardErrorTest(chex.TestCase):
         def logdensity_fn(x):
             return stats.multivariate_normal.logpdf(x, loc, cov).sum()
 
-        return logdensity_fn, loc, scale, rho
+        return logdensity_fn, loc, scale, rho, cov
 
     def mcse_test(self, samples, true_param, p_val=0.01):
         posterior_mean = jnp.mean(samples, axis=[0, 1])
@@ -554,20 +748,28 @@ class MonteCarloStandardErrorTest(chex.TestCase):
         return scaled_error
 
     @parameterized.parameters(mcse_test_cases)
-    def test_mcse(self, algorithm, parameters):
+    def test_mcse(self, algorithm, parameters, is_mass_matrix_diagonal):
         """Test convergence using Monte Carlo CLT across multiple chains."""
-        init_fn_key, pos_init_key, sample_key = jax.random.split(self.key, 3)
+        pos_init_key, sample_key = jax.random.split(self.key)
         (
             logdensity_fn,
             true_loc,
             true_scale,
             true_rho,
+            true_cov,
         ) = self.generate_multivariate_target(None)
-        kernel = algorithm(
-            logdensity_fn,
-            inverse_mass_matrix=true_scale,
-            **parameters,
-        )
+        if is_mass_matrix_diagonal is not None:
+            if is_mass_matrix_diagonal:
+                inverse_mass_matrix = true_scale**2
+            else:
+                inverse_mass_matrix = true_cov
+            kernel = algorithm(
+                logdensity_fn,
+                inverse_mass_matrix=inverse_mass_matrix,
+                **parameters,
+            )
+        else:
+            kernel = algorithm(logdensity_fn, **parameters)
 
         num_chains = 10
         initial_positions = jax.random.normal(pos_init_key, [num_chains, 2])
